@@ -32,11 +32,12 @@ from asyncio import (
     TimeoutError as AioTimeoutError,
 )
 from itertools import islice
-from threading import Lock, Thread
+from threading import Lock
 from collections import defaultdict
 from functools import partial, wraps
 from concurrent.futures import ThreadPoolExecutor
-from weakref import WeakKeyDictionary as WeakKeyDict
+from weakref import WeakKeyDictionary as WeakKeyDict, finalize
+from time import sleep
 from typing import (
     Any, AsyncIterable, Awaitable, Callable, Coroutine, DefaultDict, Dict,
     FrozenSet, Generic, Iterable, Iterator, List, Optional, Set,
@@ -676,6 +677,7 @@ def async_background_batcher(
     *,
     max_batch_size: int = ...,
     max_concurrent_batches: int = ...,
+    batch_timeout: float = ...,
 ) -> Callable[
     [_BatchFunc[A_contra, R_co]],
     _AsyncBackgroundBatcherProto[A_contra, R_co],
@@ -688,6 +690,7 @@ def async_background_batcher(
     *,
     max_batch_size: int = ...,
     max_concurrent_batches: int = ...,
+    batch_timeout: float = ...,
 ) -> _AsyncBackgroundBatcherProto[A_contra, R_co]: ...
 
 
@@ -696,6 +699,7 @@ def async_background_batcher(
     *,
     max_batch_size: int = 256,
     max_concurrent_batches: int = 5,
+    batch_timeout: float = 0.05,
 ) -> Union[
     Callable[
         [_BatchFunc[A_contra, R_co]],
@@ -751,6 +755,7 @@ def async_background_batcher(
             async_background_batcher,  # type: ignore
             max_batch_size=max_batch_size,
             max_concurrent_batches=max_concurrent_batches,
+            batch_timeout=batch_timeout,
         )
 
     batchers: 'WeakKeyDict[Loop, AsyncBackgroundBatcher[A_contra, R_co]]' \
@@ -767,6 +772,7 @@ def async_background_batcher(
                 cast(_BatchFunc[A_contra, R_co], func),
                 max_batch_size=max_batch_size,
                 max_concurrent_batches=max_concurrent_batches,
+                batch_timeout=batch_timeout,
             )
         return await batcher(arg, key=key)
 
@@ -846,6 +852,9 @@ class AsyncBackgroundBatcher(Generic[A_contra, R_co]):
     #: Maximum number of elements in each batch.
     #: Can be safely mutated after initialization.
     max_batch_size: int
+    #: Number of seconds to wait for more elements before an
+    #: incomplete batch is processed
+    batch_timeout: float
 
     #: Queue used to buffer all tasks to be executed
     _queue: 'aio.Queue[_TaskTuple[A_contra, R_co]]'
@@ -862,17 +871,22 @@ class AsyncBackgroundBatcher(Generic[A_contra, R_co]):
                  func: _BatchFunc[A_contra, R_co],
                  *,
                  max_batch_size: int = 256,
-                 max_concurrent_batches: int = 5):
+                 max_concurrent_batches: int = 5,
+                 batch_timeout: float = 0.05):
         """
         :param func: Batch execution function
         :param max_batch_size:
             Maximum size for each batch passed to `func`
         :param max_concurrent_batches:
             Maximum number of concurrent executions of `func`
+        :param batch_timeout:
+            Number of seconds to wait for more elements before an
+            incomplete batch is processed
         """
         self.func = func
         self._queue = aio.Queue()
         self.max_batch_size = max_batch_size
+        self.batch_timeout = batch_timeout
         self._semaphore = aio.Semaphore(value=max_concurrent_batches)
         self._loop = aio.get_running_loop()
         self._loop_task = self._daemon_task(self._processing_loop())
@@ -911,7 +925,13 @@ class AsyncBackgroundBatcher(Generic[A_contra, R_co]):
         """
         q = self._queue
         # Wait for the first task to appear in the queue
-        tasks = [await q.get()]
+        try:
+            tasks = [await q.get()]
+        except RuntimeError as e:
+            if str(e).lower() == 'event loop is closed':
+                return []  # Don't spam errors when shutting down
+            raise
+
         while len(tasks) < self.max_batch_size:
             # Grab all available tasks as efficiently as possible
             # Yes - this is a micro-optimization
@@ -927,7 +947,7 @@ class AsyncBackgroundBatcher(Generic[A_contra, R_co]):
             # Give the publisher a little breathing room to actually
             # populate the queue before declaring it empty
             try:
-                tasks.append(await aio.wait_for(q.get(), timeout=0.05))
+                tasks.append(await aio.wait_for(q.get(), self.batch_timeout))
             except AioTimeoutError:  # No more tasks coming
                 break
         return tasks
@@ -1032,8 +1052,9 @@ async def ensure_aw(aw: Awaitable[T], loop: Loop) -> T:
         raise RuntimeError("Target loop is closed!")
 
     def _loop_thread() -> T:
-        aio.set_event_loop(loop)
-        return loop.run_until_complete(aw)
+        with _get_loop_lock(loop):
+            aio.set_event_loop(loop)
+            return loop.run_until_complete(aw)
 
     return await main_loop.run_in_executor(_CROSS_LOOP_POOL, _loop_thread)
 
@@ -1048,7 +1069,6 @@ def loop_in_thread(loop: Loop) -> Callable[[], None]:
     >>> loop = aio.get_event_loop()
 
     >>> stop = loop_in_thread(loop)
-    >>> sleep(0.05)  # Give the loop a chance to start
     >>> loop.is_running()  # Running in background thread
     True
 
@@ -1067,24 +1087,47 @@ def loop_in_thread(loop: Loop) -> Callable[[], None]:
 
     >>> stop()  # Signal loop to stop
 
-    >>> sleep(0.05)  # Give it a little bit of time to actually stop
-
     >>> loop.is_running()  # No longer running
     False
     """
-    stop = aio.Event()
+    def _loop_thread() -> None:
+        with _get_loop_lock(loop):
+            aio.set_event_loop(loop)
+            loop.run_forever()
 
-    async def _spin() -> None:
-        aio.set_event_loop(loop)
-        await stop.wait()
+    future = _CROSS_LOOP_POOL.submit(_loop_thread)
 
-    Thread(target=loop.run_until_complete, args=(_spin(),), daemon=True).start()
+    while not loop.is_running():
+        sleep(0)  # Force switching to other threads
 
-    def _stopper() -> None: loop.call_soon_threadsafe(stop.set)
+    def _stopper() -> None:
+        loop.call_soon_threadsafe(loop.stop)
+        future.result()  # Wait for loop to exit and reveal errors
 
     return _stopper
 
 
+_LOOP_LOCKS: Dict[int, Lock] = {}
+_LOOP_LOCKS_CREATE_LOCK = Lock()
+
+
+def _get_loop_lock(loop: aio.AbstractEventLoop) -> Lock:
+    key = id(loop)
+    try:  # Check to see if lock is already created
+        return _LOOP_LOCKS[key]
+    except KeyError:
+        pass
+
+    with _LOOP_LOCKS_CREATE_LOCK:  # Ensure atomic creation
+        try:  # Handle case where another thread acquired lock first
+            return _LOOP_LOCKS[key]
+        except KeyError:  # FIRST! Create the lock
+            lock = _LOOP_LOCKS[key] = Lock()
+            # Ensure the lock is cleaned up when the loop is destroyed
+            finalize(loop, _LOOP_LOCKS.pop, key, None)  # type: ignore
+            return lock
+
+
 async def run_aw_threadsafe(aw: Awaitable[T], loop: Loop) -> T:
     """
     Thin wrapper around :func:`asyncio.run_coroutine_threadsafe` to
